@@ -8,10 +8,9 @@ from .discrete_allocation import get_latest_prices, DiscreteAllocation
 from .efficient_frontier import EfficientFrontier
 from .hierarchical_portfolio import HRPOpt
 from .risk_models import CovarianceShrinkage
-from .plotting import Plotting
 
 
-__version__ = "1.2.0"
+__version__ = "1.2.4"
 
 __all__ = [
     "market_implied_prior_returns",
@@ -23,5 +22,4 @@ __all__ = [
     "EfficientFrontier",
     "HRPOpt",
     "CovarianceShrinkage",
-    "Plotting",
 ]
