@@ -6,7 +6,7 @@ views. In addition, two utility functions are defined, which calculate:
 - market-implied prior estimate of returns
 - market-implied risk-aversion parameter
 """
-
+import sys
 import warnings
 import numpy as np
 import pandas as pd
@@ -65,6 +65,8 @@ def market_implied_risk_aversion(market_prices, frequency=252, risk_free_rate=0.
                             frequency of expected returns.
     :type risk_free_rate: float, optional
     :raises TypeError: if market_prices cannot be parsed
+    :return: market-implied risk aversion
+    :rtype: float
     """
     if not isinstance(market_prices, (pd.Series, pd.DataFrame)):
         raise TypeError("Please format market_prices as a pd.Series")
@@ -133,8 +135,9 @@ class BlackLittermanModel(base_optimizer.BaseOptimizer):
         :param cov_matrix: NxN covariance matrix of returns
         :type cov_matrix: pd.DataFrame or np.ndarray
         :param pi: Nx1 prior estimate of returns, defaults to None.
-                   Can instead pass "market" to use a market-implied prior (requires market_caps
+                   If pi="market", calculate a market-implied prior (requires market_caps
                    to be passed).
+                   If pi="equal", use an equal-weighted prior.
         :type pi: np.ndarray, pd.Series, optional
         :param absolute_views: a colleciton of K absolute views on a subset of assets,
                                defaults to None. If this is provided, we do not need P, Q.
@@ -160,6 +163,11 @@ class BlackLittermanModel(base_optimizer.BaseOptimizer):
         :param risk_free_rate: (kwarg) risk_free_rate is needed in some methods
         :type risk_free_rate: float, defaults to 0.02
         """
+        if sys.version_info[1] == 5:  # if python 3.5
+            warnings.warn(
+                "When using python 3.5 you must explicitly construct the Black-Litterman inputs"
+            )
+
         # Keep raw dataframes
         self._raw_cov_matrix = cov_matrix
 
@@ -206,12 +214,14 @@ class BlackLittermanModel(base_optimizer.BaseOptimizer):
             raise TypeError("views should be a dict or pd.Series")
         # Coerce to series
         views = pd.Series(absolute_views)
-        # Q is easy to construct
-        Q = views.values.reshape(-1, 1)
-        # P maps views to the universe.
-        P = np.zeros((len(Q), self.n_assets))
+        k = len(views)
+
+        Q = np.zeros((k, 1))
+        P = np.zeros((k, self.n_assets))
+
         for i, view_ticker in enumerate(views.keys()):
             try:
+                Q[i] = views[view_ticker]
                 P[i, list(self.tickers).index(view_ticker)] = 1
             except ValueError:
                 #  Could make this smarter by just skipping
@@ -256,6 +266,8 @@ class BlackLittermanModel(base_optimizer.BaseOptimizer):
             self.pi = market_implied_prior_returns(
                 market_caps, self.risk_aversion, self.cov_matrix, risk_free_rate
             ).reshape(-1, 1)
+        elif pi == "equal":
+            self.pi = np.ones((self.n_assets, 1)) / self.n_assets
         else:
             raise TypeError("pi must be an array or series")
 
@@ -421,7 +433,7 @@ class BlackLittermanModel(base_optimizer.BaseOptimizer):
         :param risk_aversion: risk aversion parameter, defaults to 1
         :type risk_aversion: positive float, optional
         :return: asset weights implied by returns
-        :rtype: dict
+        :rtype: OrderedDict
         """
         if risk_aversion is None:
             risk_aversion = self.risk_aversion
@@ -431,7 +443,13 @@ class BlackLittermanModel(base_optimizer.BaseOptimizer):
         b = self.posterior_rets
         raw_weights = np.linalg.solve(A, b)
         self.weights = raw_weights / raw_weights.sum()
-        return dict(zip(self.tickers, self.weights))
+        return self._make_output_weights()
+
+    def optimize(self, risk_aversion=None):
+        """
+        Alias for bl_weights for consistency with other methods.
+        """
+        return self.bl_weights(risk_aversion)
 
     def portfolio_performance(self, verbose=False, risk_free_rate=0.02):
         """
