@@ -12,13 +12,14 @@ Currently implemented:
   permission from Marcos Lopez de Prado (2016).
 """
 
+import collections
 import warnings
 import numpy as np
 import pandas as pd
 import scipy.cluster.hierarchy as sch
 import scipy.spatial.distance as ssd
 
-from . import base_optimizer
+from . import base_optimizer, risk_models
 
 
 class HRPOpt(base_optimizer.BaseOptimizer):
@@ -49,18 +50,28 @@ class HRPOpt(base_optimizer.BaseOptimizer):
     - ``save_weights_to_file()`` saves the weights to csv, json, or txt.
     """
 
-    def __init__(self, returns):
+    def __init__(self, returns=None, cov_matrix=None):
         """
         :param returns: asset historical returns
         :type returns: pd.DataFrame
+        :param cov_matrix: covariance of asset returns
+        :type cov_matrix: pd.DataFrame.
         :raises TypeError: if ``returns`` is not a dataframe
         """
-        if not isinstance(returns, pd.DataFrame):
+        if returns is None and cov_matrix is None:
+            raise ValueError("Either returns or cov_matrix must be provided")
+
+        if returns is not None and not isinstance(returns, pd.DataFrame):
             raise TypeError("returns are not a dataframe")
 
         self.returns = returns
+        self.cov_matrix = cov_matrix
         self.clusters = None
-        tickers = list(returns.columns)
+
+        if returns is None:
+            tickers = list(cov_matrix.columns)
+        else:
+            tickers = list(returns.columns)
         super().__init__(len(tickers), tickers)
 
     @staticmethod
@@ -79,9 +90,7 @@ class HRPOpt(base_optimizer.BaseOptimizer):
         cov_slice = cov.loc[cluster_items, cluster_items]
         weights = 1 / np.diag(cov_slice)  # Inverse variance weights
         weights /= weights.sum()
-        w = weights.reshape(-1, 1)
-        cluster_var = np.dot(np.dot(w.T, cov_slice), w)[0, 0]
-        return cluster_var
+        return np.linalg.multi_dot((weights, cov_slice, weights))
 
     @staticmethod
     def _get_quasi_diag(link):
@@ -93,23 +102,7 @@ class HRPOpt(base_optimizer.BaseOptimizer):
         :return: sorted list of indices
         :rtype: list
         """
-        link = link.astype(int)
-        # The new clusters formed
-        c = np.arange(link.shape[0]) + link[-1, 3]
-        root_id = c[-1]
-        d = dict(list(zip(c, link[:, 0:2].tolist())))
-
-        # Unpacks the linkage matrix recursively.
-        def recursive_unlink(curr, d):
-            """ Start this with curr = root integer """
-            if curr in d:
-                return [
-                    node for parent in d[curr] for node in recursive_unlink(parent, d)
-                ]
-            else:
-                return [curr]
-
-        return recursive_unlink(root_id, d)
+        return sch.to_tree(link, rd=False).pre_order()
 
     @staticmethod
     def _raw_hrp_allocation(cov, ordered_tickers):
@@ -151,52 +144,29 @@ class HRPOpt(base_optimizer.BaseOptimizer):
         Construct a hierarchical risk parity portfolio
 
         :return: weights for the HRP portfolio
-        :rtype: dict
+        :rtype: OrderedDict
         """
-        corr, cov = self.returns.corr(), self.returns.cov()
+        if self.returns is None:
+            cov = self.cov_matrix
+            corr = risk_models.cov_to_corr(self.cov_matrix).round(6)
+        else:
+            corr, cov = self.returns.corr(), self.returns.cov()
 
         # Compute distance matrix, with ClusterWarning fix as
         # per https://stackoverflow.com/questions/18952587/
-        dist = ssd.squareform(((1 - corr) / 2) ** 0.5)
+
+        # this can avoid some nasty floating point issues
+        matrix = np.sqrt(np.clip((1.0 - corr) / 2.0, a_min=0.0, a_max=1.0))
+        dist = ssd.squareform(matrix, checks=False)
 
         self.clusters = sch.linkage(dist, "single")
         sort_ix = HRPOpt._get_quasi_diag(self.clusters)
         ordered_tickers = corr.index[sort_ix].tolist()
         hrp = HRPOpt._raw_hrp_allocation(cov, ordered_tickers)
-        weights = dict(hrp.sort_index())
+        weights = collections.OrderedDict(hrp.sort_index())
         self.set_weights(weights)
         return weights
 
-    def plot_dendrogram(self, show_tickers=True, filename=None, showfig=True):
-        try:
-            import matplotlib.pyplot as plt
-        except (ModuleNotFoundError, ImportError):
-            raise ImportError("Please install matplotlib via pip or poetry")
-
-        warnings.warn(
-            "This method is deprecated and will be removed in v1.2.0. "
-            "Please use pypfopt.plotting instead"
-        )
-
-        if self.clusters is None:
-            self.optimize()
-
-        fig, ax = plt.subplots()
-        if show_tickers:
-            sch.dendrogram(self.clusters, labels=self.tickers, ax=ax)
-            plt.xticks(rotation=90)
-            plt.tight_layout()
-        else:
-            sch.dendrogram(self.clusters, ax=ax)
-
-        if filename:
-            plt.savefig(fname=filename, dpi=300)
-
-        if showfig:
-            plt.show()
-
-        return ax
-
     def portfolio_performance(self, verbose=False, risk_free_rate=0.02, frequency=252):
         """
         After optimising, calculate (and optionally print) the performance of the optimal
@@ -216,10 +186,13 @@ class HRPOpt(base_optimizer.BaseOptimizer):
         :return: expected return, volatility, Sharpe ratio.
         :rtype: (float, float, float)
         """
+        if self.returns is None:
+            cov = self.cov_matrix
+            mu = None
+        else:
+            cov = self.returns.cov() * frequency
+            mu = self.returns.mean() * frequency
+
         return base_optimizer.portfolio_performance(
-            self.weights,
-            self.returns.mean() * frequency,
-            self.returns.cov() * frequency,
-            verbose,
-            risk_free_rate,
+            self.weights, mu, cov, verbose, risk_free_rate
         )
